#include <iostream.h>
#include <fstream.h>
#include "ap.h"
#include "apint.h"

void message2bytes (apint m, char *b, size_t p, size_t l)
{
    size_t t;
    apdiv_t v;

    v.quot = m;

    // Slow n^2 algorithm, could be done faster, however is not significant
    for (t = p; t--;)
        b[t] = (int) ap2rawtype ((v = div (v.quot, 256)).rem.val.ap) - t - l;
}

int main (int argc, char *argv[])
{
    size_t l, p;
    apint d, pq, m, c;
    char *b;

    if (argc < 4)
    {
        cerr << "USAGE: rsadec inputfile outputfile privatekeyfile" << endl;
        return 2;
    }

    ifstream in (argv[1]);

    if (!in)
    {
        cerr << "Unable to open input file: " << argv[1] << endl;
        return 1;
    }

    ofstream out (argv[2], ios::out | ios::BIN);

    if (!out)
    {
        cerr << "Unable to open output file: " << argv[2] << endl;
        return 1;
    }

    ifstream key (argv[3], ios::in | ios::nocreate);

    if (!key)
    {
        cerr << "Unable to open private key file: " << argv[3] << endl;
        return 1;
    }

    if (!(key >> pq >> d))
    {
        cerr << "Key not found in key file: " << argv[3] << endl;
        return 1;
    }

    if (!(in >> l))
    {
        cerr << "Unknown message length." << endl;
        return 1;
    }

    cout << "Calculating block length..." << endl;
    for (p = 0, m = 0; m < pq; p++)
        m = m * 256 + 255;

    assert (p);
    p--;

    b = new char[p];

    while (l)
    {
        cout << l << " bytes left to decode...    \r";
        cout.flush ();
        if (l < p) p = l;
        if (!(in >> c)) break;
        m = powmod (c, d, pq);
        message2bytes (m, b, p, l);
        out.write (b, p);
        l -= p;
    }

    cout << "Done.                                 " << endl;

    delete[] b;

    return 0;
}
